"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FluxAccessory = void 0;
const LightState = require("node-hue-api/lib/model/lightstate/LightState");
const sleep_1 = require("./sleep");
//@ts-ignore
const suncalc_1 = require("suncalc");
const node_cron_1 = __importDefault(require("node-cron"));
const wiz_lib_1 = require("@watsonb8/wiz-lib");
const colorUtil_1 = require("./util/colorUtil");
const SECONDS_IN_DAY = 86400000;
const MINUTES_IN_MILLISECOND = 60000;
const SECONDS_IN_HOUR = 3600;
class FluxAccessory {
    _platform;
    _accessory;
    _config;
    _isActive;
    _hueRGB;
    _wizRGB;
    _fade;
    _cron;
    //Service fields
    _switchService;
    _hue;
    _lights = [];
    _wizLights = [];
    _times;
    _tasks = [];
    constructor(props) {
        //Assign class variables
        this._platform = props.platform;
        this._accessory = props.accessory;
        this._config = props.config;
        this._isActive = false;
        this._wizLights = props.wizBulbs;
        this._hue = props.hue;
        this.name = this._config.name;
        this._hueRGB = { r: 0, g: 0, b: 0 };
        this._wizRGB = { r: 0, g: 0, b: 0 };
        this._fade = this._config.transition ?? 30000;
        this._cron = this._config.cron ?? "*/30 * * * * *";
        this._times = (0, suncalc_1.getTimes)(new Date(), this._config.latitude, this._config.longitude);
        //Schedule job to refresh times
        node_cron_1.default.schedule("0 12 * * *", () => {
            this._times = (0, suncalc_1.getTimes)(new Date(), this._config.latitude, this._config.longitude);
            this._platform.log.info("Updated sunset times");
        }, {
            scheduled: true,
        }).start();
        //Schedule job to refresh hues every minute
        this.updateRGB();
        node_cron_1.default.schedule("* * * * *", () => {
            this.updateRGB();
            this._platform.log.info("Updated hues");
        }, {
            scheduled: true,
        }).start();
        this.scheduleLights();
        this._accessory
            .getService(this._platform.api.hap.Service.AccessoryInformation)
            .setCharacteristic(this._platform.api.hap.Characteristic.Manufacturer, "Brandon Watson")
            .setCharacteristic(this._platform.api.hap.Characteristic.Model, "F.lux")
            .setCharacteristic(this._platform.api.hap.Characteristic.SerialNumber, "123-456-789");
        const switchUUID = this._platform.api.hap.uuid.generate(`${this._accessory.displayName} Switch`);
        this._switchService =
            this._accessory.getService(this._platform.api.hap.Service.Switch) ||
                this._accessory.addService(this._platform.api.hap.Service.Switch, this._accessory.displayName, switchUUID);
        this._switchService
            .getCharacteristic(this._platform.api.hap.Characteristic.On)
            //@ts-ignore
            .on("set", this.onSetEnabled)
            //@ts-ignore
            .on("get", this.onGetEnabled);
        this.test();
    }
    name = "Flux";
    platformAccessory;
    /**
     * Handler for switch set event
     * @param callback The callback function to call when complete
     */
    onSetEnabled = async (activeState, callback) => {
        if (activeState) {
            this._times = (0, suncalc_1.getTimes)(new Date(), this._config.latitude, this._config.longitude);
            this._isActive = true;
            this.enable();
        }
        else {
            this._isActive = false;
            this.disable();
        }
        return callback();
    };
    /**
     * Handler for switch get event
     * @param callback The callback function to call when complete
     */
    onGetEnabled = (callback) => {
        callback(null, this._isActive);
        // return this._isActive;
    };
    /**
     * Populates internal lights array using the configuration values
     */
    getLights = async () => {
        for (const value of this._config.hueLights) {
            //@ts-ignore
            const light = await this._hue.lights.getLightByName(value.name);
            this._lights.push(light);
        }
    };
    isHueError = (object) => {
        return "_hueError" in object;
    };
    /**
     * Gets adjusted color temperature.
     */
    getTempOffset = (startTemp, endTemp, startTime, endTime) => {
        const now = this.getNow().getTime();
        const percentComplete = (now - startTime.getTime()) /
            (endTime.getTime() - startTime.getTime());
        const tempRange = Math.abs(startTemp - endTemp);
        const tempOffset = tempRange * percentComplete;
        return startTemp - tempOffset;
    };
    /**
     * Get the current time. Use test time if present.
     */
    getNow() {
        if (this._config.testNowDateString) {
            return new Date(this._config.testNowDateString);
        }
        else {
            return new Date();
        }
    }
    updateRGB = () => {
        const now = this.getNow();
        //Pad start time by an hour before sunset
        const start = new Date(this._times.sunset.getTime() - 60 * MINUTES_IN_MILLISECOND);
        const sunsetStart = this._times.sunsetStart;
        const sunsetEnd = new Date(this._times.sunset.getTime() + this._config.sunsetDuration);
        const nightStart = new Date(sunsetEnd.getTime() + 60 * MINUTES_IN_MILLISECOND);
        const sunrise = new Date(this._times.sunrise.getTime() + 1 * SECONDS_IN_DAY);
        const startColorTemp = this._config.ceilingColorTemp ?? 4000;
        const sunsetColorTemp = this._config.sunsetColorTemp ?? 2800;
        const floorColorTemp = this._config.floorColorTemp ?? 1900;
        let newTemp = this._config.ceilingColorTemp;
        if (start < now && now < sunsetStart) {
            newTemp = this.getTempOffset(startColorTemp, sunsetColorTemp, start, sunsetStart);
        }
        else if (sunsetStart < now && now < sunsetEnd) {
            newTemp = this._config.sunsetColorTemp;
        }
        else if (sunsetEnd < now && now < nightStart) {
            newTemp = this.getTempOffset(sunsetColorTemp, floorColorTemp, sunsetEnd, nightStart);
        }
        else if (nightStart < now && now < sunrise) {
            newTemp = this._config.floorColorTemp;
        }
        //Set RGB
        this._hueRGB = (0, colorUtil_1.colorTempToRgb)(newTemp);
        this._wizRGB = (0, wiz_lib_1.colorTemperature2rgb)(newTemp);
    };
    scheduleLights = async () => {
        if (this._lights.length === 0) {
            await this.getLights();
        }
        this._tasks = [...this.getHueTasks(), ...this.getWizTasks()];
    };
    getHueTasks() {
        return this._config.hueLights.map((hueLightConfig) => {
            let light = this._lights.find((x) => x.name == hueLightConfig.name);
            let schedule = hueLightConfig.cron ?? this._cron;
            this._platform.log.info(`Scheduling task for ${light?.name}: ${schedule}`);
            return node_cron_1.default.schedule(schedule, async () => {
                await this.updateHueLight(light);
                this._platform.log.info("Updated hues");
            }, {
                scheduled: false,
            });
        });
    }
    getWizTasks() {
        return this._wizLights.map((wizBulb) => {
            let wizLightConfig = this._config.wizLights.find((x) => x.ip == wizBulb.getIp());
            let schedule = wizLightConfig?.cron ?? this._cron;
            this._platform.log.info(`Scheduling task for ${wizBulb.getMac()}: ${schedule}`);
            return node_cron_1.default.schedule(schedule, async () => {
                await this.updateWizLight(wizBulb);
                this._platform.log.info("Updated hues");
            }, {
                scheduled: false,
            });
        });
    }
    updateWizLight = async (wizBulb) => {
        let pilot;
        try {
            pilot = await wizBulb.get();
        }
        catch (err) {
            this._platform.log.error(err.message);
        }
        this._platform.log.info(`Adjusting wiz bulb: ${wizBulb.getMac()}`);
        wizBulb.set(this._wizRGB, pilot?.state ? pilot.dimming : 0, this._fade);
    };
    updateHueLight = async (hueLight) => {
        if (!hueLight) {
            return;
        }
        this._platform.log.info(`Adjusting wiz bulb: ${hueLight.name}`);
        const lightState = new LightState();
        lightState
            .transitionInMillis(this._fade)
            .rgb(this._hueRGB.r ?? 0, this._hueRGB.g ?? 0, this._hueRGB.b ?? 0);
        try {
            await this._hue.lights.setLightState(hueLight.id, lightState);
        }
        catch (err) {
            if (this.isHueError(err) &&
                err.message ===
                    "parameter, xy, is not modifiable. Device is set to off.") {
                //Eat this
            }
            else {
                this._platform.log.info(`Error while setting lights: ${err}`);
            }
        }
    };
    enable() {
        this._tasks.forEach((task) => task.start());
    }
    disable() {
        this._tasks.forEach((task) => task.stop());
    }
    test = async () => {
        for (let i = 2500; i > 0; i--) {
            this._platform.log.info(`i: ${i}`);
            for (const wizBulb of this._wizLights) {
                let pilot;
                try {
                    pilot = await wizBulb.get();
                }
                catch (err) {
                    this._platform.log.error(err.message);
                }
                this._platform.log.info(`Adjusting wiz bulb: ${wizBulb.getMac()}`);
                wizBulb.set((0, wiz_lib_1.colorTemperature2rgb)(i), 100, this._fade);
            }
            await (0, sleep_1.Sleep)(100);
        }
    };
}
exports.FluxAccessory = FluxAccessory;
//# sourceMappingURL=fluxAccessory.js.map